package com.fs.starfarer.api.campaign;

import java.awt.Color;
import java.util.List;

import com.fs.starfarer.api.Script;
import com.fs.starfarer.api.campaign.comm.CommMessageAPI;
import com.fs.starfarer.api.combat.BattleCreationContext;
import com.fs.starfarer.api.loading.HullModSpecAPI;
import com.fs.starfarer.api.ui.HintPanelAPI;
import com.fs.starfarer.api.util.FaderUtil;

public interface CampaignUIAPI {
	
	public static enum CoreUITradeMode {
		OPEN,
		SNEAK,
	}
	
	MessageDisplayAPI getMessageDisplay();
	
	void addMessage(String text);
	void addMessage(String text, Color color);
	void addMessage(String text, String highlight, Color highlightColor);
	void addMessage(String text, Color color, String highlight, Color highlightColor);
	void addMessage(String text, Color color, String h1, String h2, Color hc1, Color hc2);
	void addMessage(String text, Color color, String h1, String h2, String h3, Color hc1, Color hc2, Color hc3);
	
	void clearMessages();
	
	boolean isShowingDialog();
	void startBattle(BattleCreationContext context);
	
	/**
	 * Returns true if dialog was actually shown, false otherwise (if, for example, UI is already showing another dialog).
	 * @param plugin
	 * @param interactionTarget can be null.
	 */
	boolean showInteractionDialog(InteractionDialogPlugin plugin, SectorEntityToken interactionTarget);
	
	
	/**
 	 * Returns true if dialog was actually shown, false otherwise (if, for example, UI is already showing another dialog).
	 * Picks whatever dialog is appropriate based on the various plugins that provide dialog
	 * implementations.
	 * @param interactionTarget
	 */
	boolean showInteractionDialog(SectorEntityToken interactionTarget);
	
	
	void showCoreUITab(CoreUITabId tab);
	
	/**
	 * @param tab
	 * @param custom CommMessageAPI to select in intel tab, or FleetMemberAPI to select in refit tab
	 */
	void showCoreUITab(CoreUITabId tab, Object custom);
	
	InteractionDialogAPI getCurrentInteractionDialog();

	void setDisallowPlayerInteractionsForOneFrame();

	FaderUtil getSharedFader();

	//void suppressMusic();

	float getZoomFactor();

	void suppressMusic(float maxLevel);

	boolean isShowingMenu();

	void resetViewOffset();

	void addMessage(CommMessageAPI message);

	List<HullModSpecAPI> getAvailableHullModsCopy();

	boolean isHullModAvailable(String id);

	List<String> getAvailableHullModIds();

	HintPanelAPI getHintPanel();

	void quickLoad();

	CoreUITabId getCurrentCoreTab();

	void cmdExitWithoutSaving();
	void cmdSaveAndExit();
	void cmdSettings();
	void cmdSaveCopy();
	void cmdSave();
	void cmdLoad();
	void cmdCodex();

	boolean showConfirmDialog(String message, String ok, String cancel, Script onOk, Script onCancel);
	boolean showConfirmDialog(String message, String ok, String cancel, float width, float height, Script onOk, Script onCancel);
	
}




