package com.fs.starfarer.api.impl.campaign;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FleetEncounterContextPlugin.FleetMemberData;
import com.fs.starfarer.api.campaign.FleetEncounterContextPlugin.Status;
import com.fs.starfarer.api.combat.ShipHullSpecAPI;
import com.fs.starfarer.api.combat.ShipVariantAPI;
import com.fs.starfarer.api.combat.ShieldAPI.ShieldType;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.ids.Stats;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import com.fs.starfarer.api.loading.HullModSpecAPI;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class DModManager {

//	public static final String HULLMOD_DAMAGE = "damage";
//	public static final String HULLMOD_PHASE_ALWAYS = "phaseAlways";
//	public static final String HULLMOD_DAMAGE_STRUCT = "damageStruct";
//	public static final String HULLMOD_DESTROYED_ALWAYS = "destroyedAlways";
//	public static final String HULLMOD_FIGHTER_BAY_DAMAGE = "fighterBayDamage";
//	public static final String HULLMOD_CARRIER_ALWAYS = "carrierAlways";
	

	public static boolean setDHull(ShipVariantAPI variant) {
		//if (!variant.getHullSpec().isDHull()) {
		if (!variant.isDHull()) {
			String dHullId = Misc.getDHullId(variant.getHullSpec());
			ShipHullSpecAPI dHull = Global.getSettings().getHullSpec(dHullId);
			variant.setHullSpecAPI(dHull);
			return true;
		}
		return false;
	}
	
	public static void addDMods(FleetMemberData data, boolean own, CampaignFleetAPI recoverer) {
		addDMods(data.getMember(), data.getStatus() == Status.DESTROYED, own, recoverer);
	}
	
	public static void addDMods(FleetMemberAPI member, boolean destroyed, boolean own, CampaignFleetAPI recoverer) {
		ShipVariantAPI variant = member.getVariant();
		//int original = getNumDMods(variant);
		
		if (destroyed) {
			addAllPermaModsWithTags(variant, Tags.HULLMOD_DESTROYED_ALWAYS);
//			if (own) {
//				int added = getNumDMods(variant) - original;
//				if (added > 0) return;
//			}
		}

		List<HullModSpecAPI> potentialMods = getModsWithTags(Tags.HULLMOD_DAMAGE);
		removeUnsuitedMods(variant, potentialMods);
		
		boolean hasStructDamage = getNumDMods(variant, Tags.HULLMOD_DAMAGE_STRUCT) > 0;
		if (hasStructDamage) {
			potentialMods = getModsWithoutTags(potentialMods, Tags.HULLMOD_DAMAGE_STRUCT);
		}
		
		if (variant.getHullSpec().getFighterBays() > 0) {
			potentialMods.addAll(getModsWithTags(Tags.HULLMOD_FIGHTER_BAY_DAMAGE));
		}
		if (variant.getHullSpec().getDefenseType() == ShieldType.PHASE) {
			potentialMods.addAll(getModsWithTags(Tags.HULLMOD_DAMAGE_PHASE));
		}
		
		if (variant.isCarrier()) {
			if (own || true) { // bit too harsh to always add damaged flight decks to recovered enemy carriers
				potentialMods.addAll(getModsWithTags(Tags.HULLMOD_CARRIER_ALWAYS));
			} else {
				addAllPermaModsWithTags(variant, Tags.HULLMOD_CARRIER_ALWAYS);
			}
		}
		
		removeModsAlreadyInVariant(variant, potentialMods);
		
		int num = 2 + new Random().nextInt(3);
		
		int reduction = 0;
		if (recoverer != null) {
			reduction = (int) recoverer.getStats().getDynamic().getValue(Stats.SHIP_DMOD_REDUCTION, 0);
			reduction = new Random().nextInt(reduction + 1);
		}
		
		num -= reduction;
		if (num < 1) num = 1;
		
		int already = getNumDMods(variant);
		
		int add = num - already;
		if (own) {
			add = (1 - reduction);
		}

		
		WeightedRandomPicker<HullModSpecAPI> picker = new WeightedRandomPicker<HullModSpecAPI>();
		picker.addAll(potentialMods);
		for (int i = 0; i < add && !picker.isEmpty(); i++) {
			HullModSpecAPI pick = picker.pickAndRemove();
			if (pick != null) {
				if (pick.hasTag(Tags.HULLMOD_DAMAGE_STRUCT) && getNumDMods(variant, Tags.HULLMOD_DAMAGE_STRUCT) > 0) {
					i--;
					continue;
				}
				variant.addPermaMod(pick.getId());
			}
		}
	}
	
	
	public static void addDMods(FleetMemberAPI member, boolean canAddDestroyedMods, int num, Random random) {
		if (random == null) random = new Random();
		
		ShipVariantAPI variant = member.getVariant();
		
		List<HullModSpecAPI> potentialMods = getModsWithTags(Tags.HULLMOD_DAMAGE);
		if (canAddDestroyedMods) potentialMods.addAll(getModsWithTags(Tags.HULLMOD_DESTROYED_ALWAYS));
		
		removeUnsuitedMods(variant, potentialMods);
		
		boolean hasStructDamage = getNumDMods(variant, Tags.HULLMOD_DAMAGE_STRUCT) > 0;
		if (hasStructDamage) {
			potentialMods = getModsWithoutTags(potentialMods, Tags.HULLMOD_DAMAGE_STRUCT);
		}
		
		if (variant.getHullSpec().getFighterBays() > 0) {
			potentialMods.addAll(getModsWithTags(Tags.HULLMOD_FIGHTER_BAY_DAMAGE));
		}
		if (variant.getHullSpec().getDefenseType() == ShieldType.PHASE) {
			potentialMods.addAll(getModsWithTags(Tags.HULLMOD_DAMAGE_PHASE));
		}
		
		if (variant.isCarrier()) {
			potentialMods.addAll(getModsWithTags(Tags.HULLMOD_CARRIER_ALWAYS));
		}
		
		removeModsAlreadyInVariant(variant, potentialMods);
		
		WeightedRandomPicker<HullModSpecAPI> picker = new WeightedRandomPicker<HullModSpecAPI>(random);
		picker.addAll(potentialMods);
		for (int i = 0; i < num && !picker.isEmpty(); i++) {
			HullModSpecAPI pick = picker.pickAndRemove();
			if (pick != null) {
				if (pick.hasTag(Tags.HULLMOD_DAMAGE_STRUCT) && getNumDMods(variant, Tags.HULLMOD_DAMAGE_STRUCT) > 0) {
					i--;
					continue;
				}
				variant.addPermaMod(pick.getId());
			}
		}
	}
	


	public static void removeUnsuitedMods(ShipVariantAPI variant, List<HullModSpecAPI> mods) {
		boolean phase = variant.getHullSpec().getDefenseType() == ShieldType.PHASE;
		boolean peakTime = variant.getHullSpec().getNoCRLossTime() < 10000;
		Iterator<HullModSpecAPI> iter = mods.iterator();
		while (iter.hasNext()) {
			HullModSpecAPI curr = iter.next();
			if (!peakTime && curr.hasTag(Tags.HULLMOD_PEAK_TIME)) {
				iter.remove();
				continue;
			}
			if (phase && curr.hasTag(Tags.HULLMOD_NOT_PHASE)) {
				iter.remove();
				continue;
			}
		}
	}
	public static void removeModsAlreadyInVariant(ShipVariantAPI variant, List<HullModSpecAPI> mods) {
		Iterator<HullModSpecAPI> iter = mods.iterator();
		while (iter.hasNext()) {
			HullModSpecAPI curr = iter.next();
			if (variant.hasHullMod(curr.getId())) iter.remove();
		}
	}
	
	public static void addAllPermaModsWithTags(ShipVariantAPI variant, String ... tags) {
		for (HullModSpecAPI mod : getModsWithTags(tags)) {
			variant.addPermaMod(mod.getId());
		}
	}
	
	public static List<HullModSpecAPI> getModsWithoutTags(List<HullModSpecAPI> mods, String ... tags) {
		List<HullModSpecAPI> result = new ArrayList<HullModSpecAPI>();
		OUTER: for (HullModSpecAPI mod : mods) {
			for (String tag : tags) {
				if (mod.hasTag(tag)) continue OUTER;
			}
			result.add(mod);
		}
		return result;
	}
	
	public static List<HullModSpecAPI> getModsWithTags(String ... tags) {
		List<HullModSpecAPI> result = new ArrayList<HullModSpecAPI>();
		for (HullModSpecAPI mod : Global.getSettings().getAllHullModSpecs()) {
			if (mod.getTags().containsAll(Arrays.asList(tags))) {
				result.add(mod);
			}
		}
		return result;
	}
	
	public static int getNumDMods(ShipVariantAPI variant) {
		int count = 0;
		for (String id : variant.getHullMods()) {
			if (getMod(id).hasTag(Tags.HULLMOD_DMOD)) count++;
		}
		return count;
	}
	
	public static int getNumDMods(ShipVariantAPI variant, String ... tags) {
		int count = 0;
		for (String id : variant.getHullMods()) {
			HullModSpecAPI mod = getMod(id);
			if (!mod.getTags().containsAll(Arrays.asList(tags))) continue;
			if (mod.hasTag(Tags.HULLMOD_DMOD)) count++;
		}
		return count;
	}
	
	public static HullModSpecAPI getMod(String id) {
		return Global.getSettings().getHullModSpec(id);
	}

}





