package com.fs.starfarer.api.impl.campaign.missions;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventManagerAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventPlugin;
import com.fs.starfarer.api.campaign.events.CampaignEventTarget;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.MissionCompletionRep;
import com.fs.starfarer.api.impl.campaign.ids.Events;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;

public class AnalyzeEntityMission extends BaseCampaignMission {
	
	private MarketAPI market; 
	private int reward;
	private float duration = 1f;
	private String factionId;
	
	private MissionCompletionRep repChange = null;
	private IntervalUtil randomCancel = new IntervalUtil(4, 6);

	private SectorEntityToken entity;

	
	public AnalyzeEntityMission(String marketId, String factionId, SectorEntityToken entity,
								int reward, float duration, MissionCompletionRep rep) {
		
		this.entity = entity;
		this.market = Global.getSector().getEconomy().getMarket(marketId);
		this.reward = reward;
		this.duration = duration;
		this.repChange = rep;
		
		if (factionId == null) factionId = market.getFactionId();
		this.factionId = factionId;
		
		CampaignEventManagerAPI eventManager = Global.getSector().getEventManager();
		//CampaignEventTarget target = new CampaignEventTarget(market);
		CampaignEventTarget target = new CampaignEventTarget(entity);
		target.setExtra(Misc.genUID());
		
		event = eventManager.primeEvent(target, Events.ANALYZE_ENTITY, this);
	}
	
	
	public void advance(float amount) {
		float days = Global.getSector().getClock().convertToDays(amount);
		
		randomCancel.advance(days);
		if (randomCancel.intervalElapsed()) {
			if ((float) Math.random() < 0.5f) {
				Global.getSector().getMissionBoard().removeMission(this, true);
			}
		}
	}
	
	@Override
	public String getPostingStage() {
		return super.getPostingStage();
	}


	public String getName() {
		String where = entity.getContainingLocation().getNameWithTypeIfNebula();
		return "Analyze Derelict in " + where;
	}

	public void playerAccept(SectorEntityToken entity) {
		super.playerAccept(entity);
		CampaignEventManagerAPI eventManager = Global.getSector().getEventManager();
		eventManager.startEvent(event);
	}

	public MarketAPI getMarket() {
		return market;
	}

	public CampaignEventPlugin getPrimedEvent() {
		return event;
	}
	
	public PersonAPI getImportantPerson() {
		return super.getImportantPerson();
	}

	public MissionCompletionRep getRepChange() {
		return repChange;
	}
	
	public int getReward() {
		return reward;
	}

	public float getDuration() {
		return duration;
	}

	public SectorEntityToken getEntity() {
		return entity;
	}

	public String getFactionId() {
		return factionId;
	}

}




