package com.fs.starfarer.api.impl.campaign.missions;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BaseOnMessageDeliveryScript;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.ReputationActionResponsePlugin.ReputationAdjustmentResult;
import com.fs.starfarer.api.campaign.comm.CommMessageAPI;
import com.fs.starfarer.api.campaign.comm.MessagePriority;
import com.fs.starfarer.api.campaign.events.CampaignEventTarget;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.RepActionEnvelope;
import com.fs.starfarer.api.impl.campaign.CoreReputationPlugin.RepActions;
import com.fs.starfarer.api.impl.campaign.events.BaseEventPlugin;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.ids.Strings;
import com.fs.starfarer.api.impl.campaign.rulecmd.AddRemoveCommodity;
import com.fs.starfarer.api.impl.campaign.rulecmd.salvage.special.BreadcrumbSpecial;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.Misc.Token;

public class AnalyzeEntityMissionEvent extends BaseEventPlugin {
	
	private AnalyzeEntityMission mission = null;
	private float elapsedDays = 0;
	private boolean ended = false;
	
	public void init(String type, CampaignEventTarget eventTarget) {
		super.init(type, eventTarget, false);
	}
	
	@Override
	public void setParam(Object param) {
		mission = (AnalyzeEntityMission) param;
	}

	public void startEvent() {
		super.startEvent();
		
		String stageId = "accept";
		Global.getSector().reportEventStage(this, stageId, mission.getAcceptLocation(), MessagePriority.DELIVER_IMMEDIATELY, 
				new BaseOnMessageDeliveryScript() {
					public void beforeDelivery(CommMessageAPI message) {
						message.setStarSystemId(mission.getEntity().getContainingLocation().getId());
						//message.setCenterMapOnEntity(mission.getEntity());
					}
				});
		
		mission.getEntity().getMemoryWithoutUpdate().set("$aem_target", true, mission.getDuration());
		mission.getEntity().getMemoryWithoutUpdate().set("$aem_eventRef", this, mission.getDuration());
		Misc.setFlagWithReason(mission.getEntity().getMemoryWithoutUpdate(), MemFlags.ENTITY_MISSION_IMPORTANT,
						       "aem", true, mission.getDuration());
	}
	
	private void endEvent() {
		ended = true;
		
		mission.getEntity().getMemoryWithoutUpdate().unset("$aem_target");
		mission.getEntity().getMemoryWithoutUpdate().unset("$aem_eventRef");
		
		Misc.setFlagWithReason(mission.getEntity().getMemoryWithoutUpdate(), MemFlags.ENTITY_MISSION_IMPORTANT,
							  "aem", false, 0f);
	}
	
	public void advance(float amount) {
		if (!isEventStarted()) return;
		if (isDone()) return;
		
		float days = Global.getSector().getClock().convertToDays(amount);
		
		//memory.advance(days);
		elapsedDays += days;
		
		if (mission.getDuration() - elapsedDays <= 0) {
			Global.getSector().reportEventStage(this, "failure", Global.getSector().getPlayerFleet(), MessagePriority.DELIVER_IMMEDIATELY,
					new BaseOnMessageDeliveryScript() {
						public void beforeDelivery(CommMessageAPI message) {
							ReputationAdjustmentResult result = Global.getSector().adjustPlayerReputation(
									new RepActionEnvelope(RepActions.MISSION_FAILURE, mission.getRepChange(),
														  message, null, true),
														  mission.getFactionId());
						}
					});
			endEvent();
			return;
		}
	}
	

	@Override
	public boolean callEvent(String ruleId, final InteractionDialogAPI dialog, List<Token> params, Map<String, MemoryAPI> memoryMap) {
		String action = params.get(0).getString(memoryMap);
		
		CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
		CargoAPI cargo = playerFleet.getCargo();
		
		if (action.equals("runPackage")) {
			int credits = mission.getReward();
			
//			dialog.getTextPanel().addParagraph(
//					"You tap a button to execute the black-box sensor package and wait for it to finish. " +
//					"On completion, it provides an escrow key that authorizes the agreed-upon payment " +
//					"to be transferred to your account.");
//			dialog.getTextPanel().addParagraph(
//					 "Presumably, it will transmit the scan results on a secure channel when you're near " +
//					 "a comm relay, and then delete itself from your systems.");
			
			AddRemoveCommodity.addCreditsGainText(credits, dialog.getTextPanel());
			cargo.getCredits().add(credits);
			
			Global.getSector().reportEventStage(this, "success", Global.getSector().getPlayerFleet(), MessagePriority.DELIVER_IMMEDIATELY, 
					new BaseOnMessageDeliveryScript() {
						public void beforeDelivery(CommMessageAPI message) {
							ReputationAdjustmentResult result = Global.getSector().adjustPlayerReputation(
									new RepActionEnvelope(RepActions.MISSION_SUCCESS, mission.getRepChange(),
														  message, dialog.getTextPanel(), true), 
														  mission.getFactionId());
						}
					});
			endEvent();
		}
		
		return true;
	}
	

	public Map<String, String> getTokenReplacements() {
		
		Map<String, String> map = super.getTokenReplacements();
		
		int daysLeft = (int) (mission.getDuration() - elapsedDays);
		if (daysLeft < 1) daysLeft = 1;
		String daysLeftStr = daysLeft + " days";
		if (daysLeft <= 1) {
			daysLeftStr = daysLeft + " day";
		}
		
		map.put("$sender", "Mission Board");
		map.put("$daysLeft", daysLeftStr);
		
		
		
		SectorEntityToken entity = mission.getEntity();
		
		map.put("$locationHint", "The target is " + BreadcrumbSpecial.getLocatedString(entity) + ".");
		if (entity.getCustomEntitySpec() != null) {
			map.put("$entityName", entity.getCustomEntitySpec().getNameInText());
		} else {
			map.put("$entityName", mission.getEntity().getName());
		}
		
		if (entity.getContainingLocation() != null) {
			//map.put("$systemName", entity.getContainingLocation().getNameWithTypeIfNebula());
			map.put("$systemName", entity.getContainingLocation().getNameWithLowercaseType());
		}
		
		map.put("$rewardCredits", Misc.getWithDGS((int) mission.getReward()) + Strings.C);

		
		return map;
	}

	@Override
	public String[] getHighlights(String stageId) {
		int daysLeft = (int) (mission.getDuration() - elapsedDays);
		if (daysLeft < 1) daysLeft = 1;
		
		List<String> result = new ArrayList<String>();
		
		if ("posting".equals(stageId)) {
			result.add("" + daysLeft);
			addTokensToList(result, "$rewardCredits");
		} else if ("success".equals(stageId)) {
			addTokensToList(result, "$rewardCredits");
		} else {
			addTokensToList(result, "$rewardCredits");
			result.add("" + daysLeft);
		}
				
		return result.toArray(new String[0]);
	}
	
	@Override
	public Color[] getHighlightColors(String stageId) {
		return super.getHighlightColors(stageId);
	}

	public boolean isDone() {
		return ended;
	}

	public String getEventName() {
		//return mission.getName();
		//return "Analyze derelict - " + daysLeftStr;
		
		String name = "";
		SectorEntityToken entity = mission.getEntity();
		if (entity.getCustomEntitySpec() != null) {
			name = entity.getCustomEntitySpec().getNameInText();
		} else {
			name = entity.getName();
		}
		
		int daysLeft = (int) (mission.getDuration() - elapsedDays);
		String days = "";
		if (daysLeft > 0) {
			//days = ", " + daysLeft + " days left";
			days = ", " + daysLeft + "d";
			//days = "" + daysLeft + " days";
		} else {
			days = ", <1d";
		}
		if (isDone()) {
			return "Analyze " + name + " - done"; 
		}
		return "Analyze " + name + days;
	}

	
	
	@Override
	public CampaignEventCategory getEventCategory() {
		return CampaignEventCategory.MISSION;
	}

	@Override
	public String getEventIcon() {
		//return Global.getSector().getFaction(mission.getFactionId()).getCrest()
		//return "data/missions/direstraits/icon.jpg";
		return Global.getSettings().getSpriteName("campaignMissions", "analyze_entity");
	}

	@Override
	public String getCurrentImage() {
		return Global.getSector().getFaction(mission.getFactionId()).getLogo();
	}

	
	
}



