package com.fs.starfarer.api.impl.campaign.rulecmd;

import java.util.List;
import java.util.Map;
import java.util.Random;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.Script;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.rules.MemKeys;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.characters.CharacterCreationData;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Personalities;
import com.fs.starfarer.api.impl.campaign.ids.Ranks;
import com.fs.starfarer.api.impl.campaign.ids.Skills;
import com.fs.starfarer.api.impl.campaign.tutorial.CampaignTutorialScript;
import com.fs.starfarer.api.impl.campaign.tutorial.TutorialMissionEvent;
import com.fs.starfarer.api.util.Misc.Token;

/**
 *	$ngcAddOfficer
 *	$ngcSkipTutorial
 *
 */
public class NGCAddStandardStartingScript extends BaseCommandPlugin {

	public boolean execute(String ruleId, InteractionDialogAPI dialog, List<Token> params, Map<String, MemoryAPI> memoryMap) {
		if (dialog == null) return false;
		
		CharacterCreationData data = (CharacterCreationData) memoryMap.get(MemKeys.LOCAL).get("$characterData");
		final MemoryAPI memory = memoryMap.get(MemKeys.LOCAL);
		data.addScript(new Script() {
			public void run() {
				CampaignFleetAPI fleet = Global.getSector().getPlayerFleet();
				// add crew, supplies, and fuel
				int crew = 0;
				int supplies = 0;
				for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
					crew += Math.ceil(member.getMinCrew() + (member.getMaxCrew() - member.getMinCrew()) * 0.5f);
					supplies += member.getDeploymentCostSupplies() * 4f;
				}
				
				CargoAPI cargo = fleet.getCargo();
				cargo.addCrew(crew);
				cargo.addSupplies(10);
				cargo.addFuel(cargo.getMaxFuel() * 0.5f);
				
				
				boolean addOfficer = memory.getBoolean("$ngcAddOfficer");
				boolean skipTutorial = memory.getBoolean("$ngcSkipTutorial");

				fleet.getFleetData().ensureHasFlagship();
				
				if (addOfficer) {
					for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
						if (!member.isFlagship()) {
							//PersonAPI officer = OfficerManagerEvent.createOfficer(Global.getSector().getPlayerFaction(), 1, true, SkillPickPreference.NON_CARRIER);
							PersonAPI officer = Global.getSector().getPlayerFaction().createRandomPerson(new Random());
							officer.getStats().setSkillLevel(Skills.IMPACT_MITIGATION, 1);
							officer.getStats().setSkillLevel(Skills.DAMAGE_CONTROL, 1);
							officer.setRankId(Ranks.SPACE_LIEUTENANT);
							officer.setPostId(Ranks.POST_MERCENARY);
							officer.setPersonality(Personalities.STEADY);
							officer.getStats().refreshCharacterStatsEffects();
							
							member.setCaptain(officer);
							fleet.getFleetData().addOfficer(officer);
							break;
						}
					}
				}
				
				for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
					float max = member.getRepairTracker().getMaxCR();
					member.getRepairTracker().setCR(max);
				}
				fleet.getFleetData().setSyncNeeded();
				
				if (skipTutorial) {
					StarSystemAPI system = Global.getSector().getStarSystem("galatia");
					PlanetAPI ancyra = (PlanetAPI) system.getEntityById("ancyra");
					PersonAPI mainContact = TutorialMissionEvent.createMainContact(ancyra);
					PersonAPI jangalaContact = TutorialMissionEvent.getJangalaContact();
					
					TutorialMissionEvent.endGalatiaPortionOfMission();
					
					mainContact.getRelToPlayer().setRel(0.2f);
					jangalaContact.getRelToPlayer().setRel(0.1f);
					Global.getSector().getFaction(Factions.HEGEMONY).getRelToPlayer().setRel(0.15f);
					
					float freeCargo = cargo.getSpaceLeft();
					float addMachinery = Math.min(freeCargo, 15);
					if (addMachinery > 0) {
						cargo.addCommodity(Commodities.HEAVY_MACHINERY, addMachinery);
						freeCargo -= addMachinery;
					}
					float addSupplies = Math.min(freeCargo, 70); 
					if (addSupplies > 0) {
						cargo.addSupplies(addSupplies);
						freeCargo -= addSupplies;
					}
					
					float addFuel = Math.min(cargo.getFreeFuelSpace() - 10f, 70);
					if (addFuel > 0) {
						cargo.addFuel(addFuel);
					}
					
					float addCrew = Math.min(cargo.getFreeCrewSpace() - 10f, 100f);
					if (addCrew > 0) {
						cargo.addCrew((int)addCrew);
					}
					
					cargo.getCredits().add(30000);
				} else {
					fleet.clearAbilities();
					fleet.setTransponderOn(false);
					
					StarSystemAPI system = Global.getSector().getStarSystem("galatia");
					system.addScript(new CampaignTutorialScript(system));
				}
			}
			
		});
		return true;
	}
	
	

}



