package com.fs.starfarer.api.impl.campaign.rulecmd.salvage;

import java.util.List;
import java.util.Map;
import java.util.Random;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.GenericPluginManagerAPI.GenericPlugin;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.rules.MemKeys;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.BaseGenericPlugin;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV2;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParams;
import com.fs.starfarer.api.impl.campaign.ids.Entities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.procgen.DefenderDataOverride;
import com.fs.starfarer.api.impl.campaign.procgen.SalvageEntityGenDataSpec;
import com.fs.starfarer.api.impl.campaign.procgen.themes.SalvageEntityGeneratorOld;
import com.fs.starfarer.api.impl.campaign.rulecmd.BaseCommandPlugin;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.Misc.Token;

/**
 * Actually just generates automated defenses, not salvage. SalvageEntity does that.
 * 
 * SalvageGenFromSeed
 */
public class SalvageGenFromSeed extends BaseCommandPlugin {

	public static class SDMParams {
		public SectorEntityToken entity;
		public String factionId;
		public SDMParams() {
		}
	}
	
	
	public static interface SalvageDefenderModificationPlugin extends GenericPlugin {
		float getStrength(SDMParams p, float strength, Random random, boolean withOverride);
		float getProbability(SDMParams p, float probability, Random random, boolean withOverride);
		float getMaxSize(SDMParams p, float maxSize, Random random, boolean withOverride);
		
		void modifyFleet(SDMParams p, CampaignFleetAPI fleet, Random random, boolean withOverride);
		void reportDefeated(SDMParams p, SectorEntityToken entity, CampaignFleetAPI fleet);
	}
	
	public static final String DEFEATED_DERELICT_STR = "$defeatedDerelictStr";
	public static final float DEFEATED_TO_ADDED_FACTOR = 0.2f;
	
	public static class SalvageDefenderModificationPluginImpl extends BaseGenericPlugin implements SalvageDefenderModificationPlugin {
		public float getStrength(SDMParams p, float strength, Random random, boolean withOverride) {
			if (withOverride) return strength;
			float bonus = Global.getSector().getMemory().getFloat(DEFEATED_DERELICT_STR) * DEFEATED_TO_ADDED_FACTOR;
			
			String type = p.entity.getCustomEntityType();
			float limit = 100f;
			if (Entities.DERELICT_SURVEY_PROBE.equals(type)) {
				limit = 20;
			} else if (Entities.DERELICT_SURVEY_SHIP.equals(type)) {
				limit = 30;
			} else if (Entities.DERELICT_MOTHERSHIP.equals(type)) {
				limit = 50;
			}
			
			if (bonus > limit) bonus = limit;
			return strength + (int) bonus;
		}
		public float getMaxSize(SDMParams p, float maxSize, Random random, boolean withOverride) {
			if (withOverride) return maxSize;
			
			float bonus = Global.getSector().getMemory().getFloat(DEFEATED_DERELICT_STR) * DEFEATED_TO_ADDED_FACTOR;
			String type = p.entity.getCustomEntityType();
			float bonusSize = 1;
			if (Entities.DERELICT_SURVEY_PROBE.equals(type)) {
				if (bonus >= 5) bonusSize = 2;
			}
			
			return Math.max(maxSize, bonusSize);
		}
		public float getProbability(SDMParams p, float probability, Random random, boolean withOverride) {
			if (withOverride) return probability;
			return probability;
		}
		public void reportDefeated(SDMParams p, SectorEntityToken entity, CampaignFleetAPI fleet) {
			float total = Global.getSector().getMemory().getFloat(DEFEATED_DERELICT_STR);
			for (FleetMemberAPI member : Misc.getSnapshotMembersLost(fleet)) {
				total += FleetFactoryV2.getMemberWeight(member);
			}
			Global.getSector().getMemory().set(DEFEATED_DERELICT_STR, total);
		}
		public void modifyFleet(SDMParams p, CampaignFleetAPI fleet, Random random, boolean withOverride) {
		}
		@Override
		public int getHandlingPriority(Object params) {
			if (!(params instanceof SDMParams)) return 0;
			SDMParams p = (SDMParams) params;
			
			if (Factions.DERELICT.equals(p.factionId)) {
				return 1;
			}
			
			return 0;
		}
	}
	
	public boolean execute(String ruleId, InteractionDialogAPI dialog, List<Token> params, final Map<String, MemoryAPI> memoryMap) {
		if (dialog == null) return false;
		
		SectorEntityToken entity = dialog.getInteractionTarget();
		SalvageEntityGenDataSpec spec = SalvageEntityGeneratorOld.getSalvageSpec(entity.getCustomEntityType());
		
		MemoryAPI memory = memoryMap.get(MemKeys.LOCAL);
		if (memoryMap.containsKey(MemKeys.ENTITY)) {
			memory = memoryMap.get(MemKeys.ENTITY);
		}
		
		long seed = memory.getLong(MemFlags.SALVAGE_SEED);
//		if (Global.getSettings().isDevMode()) {
//			seed = new Random().nextLong();
//		}
		
		// don't use seed directly so that results from different uses of it aren't tied together
		Random random = Misc.getRandom(seed, 0);
		
		DefenderDataOverride override = null;
		if (memory.contains(MemFlags.SALVAGE_DEFENDER_OVERRIDE)) {
			override = (DefenderDataOverride) memory.get(MemFlags.SALVAGE_DEFENDER_OVERRIDE);
		}
		
		Random fleetRandom = Misc.getRandom(seed, 1);
		float strength = spec.getMinStr() + Math.round((spec.getMaxStr() - spec.getMinStr()) * fleetRandom.nextFloat());
		float prob = spec.getProbDefenders();
		
		if (override != null) {
			strength = override.minStr + Math.round((override.maxStr - override.minStr) * fleetRandom.nextFloat());
			prob = override.probDefenders;
		}
		
		String factionId = entity.getFaction().getId();
		if (spec.getDefFaction() != null) {
			factionId = spec.getDefFaction();
		}
		if (override != null && override.defFaction != null) {
			factionId = override.defFaction;
		}
		
		SDMParams p = new SDMParams();
		p.entity = entity;
		p.factionId = factionId;
		
		SalvageDefenderModificationPlugin plugin = Global.getSector().getGenericPlugins().pickPlugin(
								SalvageDefenderModificationPlugin.class, p);
		
		if (plugin != null) {
			strength = plugin.getStrength(p, strength, random, override != null);
			prob = plugin.getProbability(p, prob, random, override != null);
		}
		
		if ((int) strength > 0 && //false &&
				random.nextFloat() < prob && 
				!memory.getBoolean("$defenderFleetDefeated")) {
			memory.set("$hasDefenders", true, 0);
			
			if (!memory.contains("$defenderFleet")) {
				MarketAPI market = Global.getFactory().createMarket("temp", "Temp", 1);
				
				market.setFactionId(factionId);
				
				
				FleetParams fleetParams = new FleetParams(
						null,
						market, 
						market.getFactionId(),
						null, // fleet's faction, if different from above, which is also used for source market picking
						FleetTypes.PATROL_SMALL,
						(int) strength, // combatPts
						0, // freighterPts 
						0, // tankerPts
						0f, // transportPts
						0f, // linerPts
						0f, // civilianPts 
						0f, // utilityPts
						0f, // qualityBonus
						1f, // qualityOverride
						0f, // officer num mult
						0 // officer level bonus
					);
				fleetParams.random = fleetRandom;
				fleetParams.withOfficers = false;
				fleetParams.maxShipSize = (int) spec.getMaxDefenderSize();
				if (override != null) {
					fleetParams.maxShipSize = override.maxDefenderSize;
				}
				
				if (plugin != null) {
					fleetParams.maxShipSize = (int) (plugin.getMaxSize(p, fleetParams.maxShipSize, random, override != null));
				}
				
				CampaignFleetAPI defenders = FleetFactoryV2.createFleet(fleetParams);
				//defenders.setName(entity.getName() + ": " + "Automated Defenses");
				defenders.setName("Automated Defenses");
				
				float probStation = spec.getProbStation();
				if (override != null) {
					probStation = override.probStation;
				}
				if (fleetRandom.nextFloat() < probStation) {
					String stationRole = spec.getStationRole();
					if (override != null && override.stationRole != null) {
						stationRole = override.stationRole;
					}
					market.getFaction().pickShipAndAddToFleet(stationRole, 1f, defenders, fleetRandom);
					defenders.getFleetData().sort();
				}
				
				defenders.clearAbilities();
				
				if (plugin != null) {
					plugin.modifyFleet(p, defenders, fleetRandom, override != null);
				}
				
				defenders.getFleetData().sort();
				
				memory.set("$defenderFleet", defenders, 0);
			}
			
			
			CampaignFleetAPI defenders = memory.getFleet("$defenderFleet");
			if (defenders != null) {
				boolean hasStation = false;
				boolean hasNonStation = false;
				for (FleetMemberAPI member : defenders.getFleetData().getMembersListCopy()) {
					if (member.isStation()) {
						hasStation = true;
					} else {
						hasNonStation = true;
					}
				}
				memory.set("$hasStation", hasStation, 0);
				memory.set("$hasNonStation", hasNonStation, 0);
				
				defenders.setLocation(entity.getLocation().x, entity.getLocation().y);
			}
		} else {
			memory.set("$hasDefenders", false, 0);
			memory.set("$hasStation", false, 0);
		}
		
		//memory.set("$hasSalvageSpecial", false, 0);
		//memory.set("$salvageSpecialData", null, 0);
		//memory.set("$salvageSpecialData", new DomainSurveyDerelictSpecialData(SpecialType.SCRAMBLED), 0);
		
	
		return true;
	}

	
}




