package com.fs.starfarer.api.impl.campaign.rulecmd.salvage.special;

import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignTerrainAPI;
import com.fs.starfarer.api.campaign.CustomCampaignEntityAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.comm.CommMessageAPI;
import com.fs.starfarer.api.impl.campaign.ids.Entities;
import com.fs.starfarer.api.impl.campaign.ids.Terrain;
import com.fs.starfarer.api.impl.campaign.procgen.Constellation;
import com.fs.starfarer.api.impl.campaign.rulecmd.salvage.SalvageSpecialInteraction.SalvageSpecialData;
import com.fs.starfarer.api.impl.campaign.rulecmd.salvage.SalvageSpecialInteraction.SalvageSpecialPlugin;
import com.fs.starfarer.api.util.FleetLog;
import com.fs.starfarer.api.util.Misc;

public class BreadcrumbSpecial extends BaseSalvageSpecial {

	
	public static class BreadcrumbSpecialData implements SalvageSpecialData {
		public String targetId = null;
		public String targetName = null;
		public BreadcrumbSpecialData(String targetId) {
			this.targetId = targetId;
		}
		
		public SalvageSpecialPlugin createSpecialPlugin() {
			return new BreadcrumbSpecial();
		}
	}
	
	private BreadcrumbSpecialData data;
	
	public BreadcrumbSpecial() {
	}
	

	@Override
	public void init(InteractionDialogAPI dialog, Object specialData) {
		super.init(dialog, specialData);
		
		data = (BreadcrumbSpecialData) specialData;
		
		initEntityLocation();
	}

	private void initEntityLocation() {
		if (data.targetId == null) {
			initNothing();
			return;
		}
		
		SectorEntityToken target = Global.getSector().getEntityById(data.targetId);
		if (target == null) { 
			initNothing();
			return;
		}
		
		// already discovered
		if (!target.hasSensorProfile() && !target.isDiscoverable()) {
			initNothing();
			return;
		}
		
		
		String targetName = getNameWithAOrAn(target, data.targetName);
		//String entityName = getNameWithAOrAn(entity, null);
		String located = getLocatedString(target);
		
		
		String subject = getString(Misc.ucFirst(targetName.substring(targetName.indexOf(" ") + 1)) + " location");
		
		
		String text1 = "The $shortName's memory banks are partially accessible, and ";
		text1 += "contain information indicating that " + targetName + " is " + located + ".";
		
		boolean debris = Entities.DEBRIS_FIELD_SHARED.equals(entity.getCustomEntityType());
		if (debris) {
			text1 = "Your salvage crews find a functional memory bank in the debris. It contains information " +
					"indicating that " + targetName.toLowerCase() + " is " + located + ".";
		}
		
		addText(text1);
		
		CommMessageAPI message = FleetLog.beginEntry(subject, target);
		message.getSection1().addPara(getString(text1));
		FleetLog.addToLog(message, text);
		
		//unsetData();
		setDone(true);
		//setShowAgain(true);
	}
	
	public static String getNameWithAOrAn(SectorEntityToken target, String override) {
		String targetAOrAn = "a";
		String targetName = override;
		if (targetName == null) {
			if (target instanceof CustomCampaignEntityAPI) {
				CustomCampaignEntityAPI custom = (CustomCampaignEntityAPI) target;
				targetName = custom.getCustomEntitySpec().getNameInText();
				targetAOrAn = custom.getCustomEntitySpec().getAOrAn();
			} else if (target instanceof PlanetAPI) {
				PlanetAPI planet = (PlanetAPI) target;
				targetName = planet.getTypeNameWithLowerCaseWorld().toLowerCase();
				targetAOrAn = planet.getSpec().getAOrAn();
			} else {
				targetName = target.getName();
			}
		}
		return targetAOrAn + " " + targetName;
	}
	
	public static String getLocatedString(SectorEntityToken target) {
		String loc = getLocationDescription(target);
		
		String orbiting = "";
		boolean useTerrain = false;
		if (target.getOrbitFocus() != null) {
			if (target.getOrbitFocus() instanceof PlanetAPI) {
				PlanetAPI focus = (PlanetAPI) target.getOrbitFocus();
				if (!focus.isStar()) {
					orbiting = "orbiting " + focus.getSpec().getAOrAn() + " " + focus.getTypeNameWithLowerCaseWorld().toLowerCase() + " in ";
				} else {
					float dist = Misc.getDistance(focus.getLocation(), target.getLocation());
					if (dist < 3000) {
						orbiting = "located in the heart of ";
					} else if (dist > 8000) {
						orbiting = "located in the outer reaches of ";
					} else {
						orbiting = "located in ";
					}
					useTerrain = true;
				}
			} else if (target.getOrbitFocus() instanceof CustomCampaignEntityAPI) {
				CustomCampaignEntityAPI custom = (CustomCampaignEntityAPI) target.getOrbitFocus();
				orbiting = "orbiting " + custom.getCustomEntitySpec().getAOrAn() + " " + custom.getCustomEntitySpec().getNameInText() + " in ";
			} else if (target.getOrbitFocus() instanceof JumpPointAPI) {
				orbiting = "orbiting a jump-point in ";
			} else { // center of a binary/nebula/etc
				//float dist = Misc.getDistance(target.getOrbitFocus().getLocation(), target.getLocation());
				float dist = Misc.getDistance(new Vector2f(), target.getLocation());
				if (dist < 3000) {
					orbiting = "located in the heart of ";
				} else if (dist > 8000) {
					orbiting = "located in the outer reaches of ";
				} else {
					orbiting = "located in ";
				}
				useTerrain = true;
			}
		} else if (target.getContainingLocation() != null && target.getContainingLocation().isNebula()) {
			float dist = Misc.getDistance(new Vector2f(), target.getLocation());
			if (dist < 3000) {
				orbiting = "located in the heart of ";
			} else if (dist > 8000) {
				orbiting = "located on the outskirts of ";
			} else {
				orbiting = "located in ";
			}
			useTerrain = true;
		}
		
		if (useTerrain) {
			String terrainString = getTerrainString(target);
			if (terrainString != null) {
				orbiting = "located in " + terrainString + " in ";
			}
		}
		
		if (orbiting == null || orbiting.isEmpty()) orbiting = "located in ";
		return orbiting + loc;
	}
	
	public static String getTerrainString(SectorEntityToken entity) {
		if (!(entity.getContainingLocation() instanceof StarSystemAPI)) return null;
		
		StarSystemAPI system = (StarSystemAPI) entity.getContainingLocation();
		for (CampaignTerrainAPI terrain : system.getTerrainCopy()) {
			if (!terrain.getPlugin().containsEntity(entity)) continue;
			
			String type = terrain.getType();
			
			if (Terrain.ASTEROID_BELT.equals(type)) return "an asteroid belt";
			if (Terrain.ASTEROID_FIELD.equals(type)) return "an asteroid field";
			//if (Terrain.MAGNETIC_FIELD.equals(type)) return "a magnetic field";
			if (Terrain.RING.equals(type)) return "a ring system";
		}
		
		return null;
	}

	public static String getLocationDescription(SectorEntityToken entity) {
		LocationAPI loc = entity.getContainingLocation();
		if (loc == null) {
			return "an unknown location"; 
		}
		if (loc.isHyperspace()) {
			return "hyperspace";
		}
		StarSystemAPI system = (StarSystemAPI) loc;
		
		if (system == Global.getSector().getCurrentLocation()) {
			return "the " + system.getNameWithLowercaseType();
		}

		if (entity.getConstellation() != null && entity.getConstellation() != Global.getSector().getCurrentLocation().getConstellation()) {
			Constellation c = entity.getConstellation();
			String cText = "in the " + c.getNameWithLowercaseType();
			if (c.getSystems().size() == 1) {
				return "the " + system.getNameWithLowercaseType();
			}
			
			if (system.isNebula()) {
				return "a nebula " + cText;
			}
			
			if (system.getTertiary() != null) {
				return "a trinary star system " + cText;
			}
			
			if (system.getSecondary() != null) {
				return "a binary star system " + cText;
			}
			
			//if (system.getType() == StarSystemType.SINGLE) {
			return "the " + system.getNameWithLowercaseType() + " " + cText;
			//return "orbit around " + getStarDescription(system.getStar()) + " " + cText;
		}
		
		
		return "the " + system.getNameWithLowercaseType();
	}
	
	/*
	public static String getStarDescription(PlanetAPI star) {
		String type = star.getTypeId();
		
		if (type.equals(StarTypes.BLACK_HOLE)) return "a black hole";
		if (type.equals(StarTypes.NEUTRON_STAR)) return "a neutron star";
		
		if (type.equals(StarTypes.ORANGE) ||
			type.equals(StarTypes.ORANGE_GIANT)) {
			return "an orange star";
		}
		
		if (type.equals(StarTypes.RED_DWARF) ||
			type.equals(StarTypes.RED_SUPERGIANT) ||
				type.equals(StarTypes.RED_GIANT)) {
			return "a red star";
		}
		
		if (type.equals(StarTypes.BLUE_GIANT) ||
				type.equals(StarTypes.BLUE_SUPERGIANT)) {
			return "a blue star";
		}
		
		if (type.equals(StarTypes.BROWN_DWARF) ||
				type.equals(StarTypes.WHITE_DWARF)) {
			return "a dim star";
		}
		
		if (type.equals(StarTypes.YELLOW)) {
			return "a yellow star";
		}
		
		return "a star of unknown type";
	}
	*/
	
	@Override
	public void optionSelected(String optionText, Object optionData) {
		super.optionSelected(optionText, optionData);
	}

	
}



