package com.fs.starfarer.api.impl.campaign.shared;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.MarketConnectionAPI;

public class SectorActivityTracker {

	private Map<String, StarSystemActivityTracker> trackers = new LinkedHashMap<String, StarSystemActivityTracker>();
	private Map<String, MarketConnectionActivityData> connectionData = new LinkedHashMap<String, MarketConnectionActivityData>();
	
	private CommodityStatTracker commodityTracker = new CommodityStatTracker();

	Object readResolve() {
		if (commodityTracker == null) {
			commodityTracker = new CommodityStatTracker();
		}
		return this;
	}
	
	Object writeReplace() {
		
		List<String> remove = new ArrayList<String>();
		for (String key : connectionData.keySet()) {
			MarketConnectionActivityData data = connectionData.get(key);
			if (data.getSmugglingPointsLost().getAverage() < 1f &&
					data.getSmugglingPointsSent().getAverage() < 1f &&
					data.getTradePointsLost().getAverage() < 1f &&
					data.getTradePointsSent().getAverage() < 1f) {
				remove.add(key);
			}
		}
		
		for (String key : remove) {
			connectionData.remove(key);
		}
		return this;
	}
	
	public void advance(float days) {
		for (StarSystemAPI system : Global.getSector().getStarSystems()) {
			getTracker(system);
		}
		for (StarSystemActivityTracker tracker : trackers.values()) {
			tracker.advance(days);
		}
		
		for (MarketConnectionActivityData data : connectionData.values()) {
			data.advance(days);
		}
		
		commodityTracker.advance(days);
	}
	
	public StarSystemActivityTracker getTracker(String starId) {
		StarSystemAPI system = Global.getSector().getStarSystem(starId);
		return getTracker(system);
	}
	public StarSystemActivityTracker getTracker(StarSystemAPI system) {
		StarSystemActivityTracker tracker = trackers.get(system.getId());
		if (tracker == null) {
			tracker = new StarSystemActivityTracker(system);
			trackers.put(system.getId(), tracker);
		}
		return tracker;
	}
	
	public MarketConnectionActivityData getConnectionData(MarketAPI one, MarketAPI two) {
		return getConnectionData(Global.getSector().getEconomy().getConnection(one, two));
	}
	
	public boolean hasConnectionData(MarketAPI one, MarketAPI two) {
		return hasConnectionData(Global.getSector().getEconomy().getConnection(one, two));
	}
	
	public boolean hasConnectionData(MarketConnectionAPI conn) {
		return connectionData.containsKey(conn.getId());
	}
	public MarketConnectionActivityData getConnectionData(MarketConnectionAPI conn) {
		MarketConnectionActivityData data = connectionData.get(conn.getId());
		if (data == null) {
			data = new MarketConnectionActivityData(conn.getId());
			connectionData.put(conn.getId(), data);
		}
		return data;
	}

	public CommodityStatTracker getCommodityTracker() {
		return commodityTracker;
	}
	
	
}








