package com.fs.starfarer.api.impl.campaign.submarkets;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetDataAPI;
import com.fs.starfarer.api.campaign.PlayerMarketTransaction;
import com.fs.starfarer.api.campaign.SubmarketPlugin;
import com.fs.starfarer.api.campaign.CampaignUIAPI.CoreUITradeMode;
import com.fs.starfarer.api.campaign.CargoAPI.CargoItemType;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.combat.WeaponAPI.AIHints;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.fleet.ShipRolePick;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import com.fs.starfarer.api.loading.FighterWingSpecAPI;
import com.fs.starfarer.api.loading.HullModSpecAPI;
import com.fs.starfarer.api.loading.WeaponSlotAPI;
import com.fs.starfarer.api.loading.WeaponSpecAPI;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class BaseSubmarketPlugin implements SubmarketPlugin {

	public static class ShipSalesData {
		private String variantId;
		private float numShips;
		private float totalValue;
		public String getVariantId() {
			return variantId;
		}
		public void setVariantId(String variantId) {
			this.variantId = variantId;
		}
		public float getNumShips() {
			return numShips;
		}
		public void setNumShips(float numShips) {
			this.numShips = numShips;
		}
		public float getTotalValue() {
			return totalValue;
		}
		public void setTotalValue(float totalValue) {
			this.totalValue = totalValue;
		}
	}
	
	protected MarketAPI market;
	protected SubmarketAPI submarket;
	
	private CargoAPI cargo;
	protected float minCargoUpdateInterval = 30; // campaign days
	protected float sinceLastCargoUpdate = minCargoUpdateInterval + 1;
	
	protected Random itemGenRandom = new Random();
	
	public void init(SubmarketAPI submarket) {
		this.submarket = submarket;
		this.market = submarket.getMarket();
	}

	public String getName() {
		return null;
	}

	public CargoAPI getCargo() {
		if (cargo == null) {
			this.cargo = Global.getFactory().createCargo(true);
			this.cargo.initMothballedShips(submarket.getFaction().getId());
		}
		return cargo;
	}
	
	public CargoAPI getCargoNullOk() {
		return cargo;
	}
	

	public void updateCargoPrePlayerInteraction() {
		
	}
	
	public void advance(float amount) {
		float days = Global.getSector().getClock().convertToDays(amount);
		sinceLastCargoUpdate += days;
		
	}
	
	public boolean okToUpdateCargo() {
		return sinceLastCargoUpdate >= minCargoUpdateInterval;
	}
	
	
	public void updateEconomicCommoditiesInCargo(boolean withMinEssential) {
		CargoAPI cargo = getCargo();
		
		for (CommodityOnMarketAPI com : market.getAllCommodities()) {
			if (com.isNonEcon()) continue;
			
			String id = com.getId();
//			if (Commodities.FUEL.equals(id)) {
//				System.out.println("efwefwe");
//			}
//			if (market.isIllegal(id)) continue;
			//float desired = computeStockpileAvailableForSale(com) * openMarketFraction;
			//float desired = com.getMaxPlayerFacingStockpile(openMarketFraction, com.getAverageStockpileAfterDemand());
			//float desired = com.getAboveDemandStockpile() * 0.4f;
			float desired = getActualAboveDemandCommodityQuantity(com) +
							getActualBelowDemandCommodityQuantity(com);
			desired = (int) desired;
			
			if (okToUpdateCargo() || cargo.getCommodityQuantity(id) <= 0) {
				float minToBotherWith = com.getCommodity().getEconUnit() * 0.1f;
				if (desired < minToBotherWith) desired = 0;
			}
			
			float current = cargo.getQuantity(CargoItemType.RESOURCES, id);
			
			float minimumToReduceTo = 0f;
			if (withMinEssential) {
				if (id.equals(Commodities.FUEL)) {
					minimumToReduceTo = Math.max(desired, Global.getSettings().getFloat("blackMarketMinFuel"));
				} else if (id.equals(Commodities.SUPPLIES)) {
					minimumToReduceTo = Math.max(desired, Global.getSettings().getFloat("blackMarketMinSupplies"));
				} else if (id.equals(Commodities.MARINES)) {
					minimumToReduceTo = Math.max(desired, Global.getSettings().getFloat("blackMarketMinMarines"));
				}
				if (okToUpdateCargo() || true) { // moved this to only be called if it's ok to update cargo
					desired = Math.max(desired, minimumToReduceTo);
				}
				
				if (desired > com.getStockpile()) {
					com.setStockpile(desired);
				}
			}
			
			if (desired > current) {
				cargo.addItems(CargoItemType.RESOURCES, id, (int)(desired - current));
			} else if (current > desired) {
				cargo.removeItems(CargoItemType.RESOURCES, id, (int)(current - Math.max(desired, minimumToReduceTo)));
			}
		}
	}
	
	public float getActualAboveDemandCommodityQuantity(CommodityOnMarketAPI com) {
		float quantity = com.getAboveDemandStockpile();
		if (quantity <= 0) return 0f;
		
		float ours = 0f;
		float total = 0f;
		
		for (SubmarketAPI sub : market.getSubmarketsCopy()) {
			float curr = sub.getPlugin().getDesiredAboveDemandCommodityQuantity(com);
			if (sub.getPlugin() == this) {
				ours += curr;
			}
			total += curr;
		}
		if (total <= 0f) return 0f;
		
		return (int) Math.min(ours, quantity * ours / total);
	}
	
	public float getActualBelowDemandCommodityQuantity(CommodityOnMarketAPI com) {
		float quantity = Math.max(0, com.getStockpile() - com.getAboveDemandStockpile());
		if (quantity <= 0) return 0f;
		
		float ours = 0f;
		float total = 0f;
		
		for (SubmarketAPI sub : market.getSubmarketsCopy()) {
			float curr = sub.getPlugin().getDesiredBelowDemandCommodityQuantity(com);
			if (sub.getPlugin() == this) {
				ours += curr;
			}
			total += curr;
		}
		if (total <= 0f) return 0f;
		
		return (int) Math.min(ours, quantity * ours / total);
	}
	
	public float getDesiredAboveDemandCommodityQuantity(CommodityOnMarketAPI com) {
		return 0f;
	}
	
	public float getDesiredBelowDemandCommodityQuantity(CommodityOnMarketAPI com) {
		return 0f;
	}
	
	
	public void addAllCargo(CargoAPI otherCargo) {
		for (CargoStackAPI stack : otherCargo.getStacksCopy()) {
			if (stack.isNull()) continue;
			getCargo().addItems(stack.getType(), stack.getData(), stack.getSize());
		}
	}
	

	public float getTariff() {
		return market.getTariff().getModifiedValue();
	}

	public String getBuyVerb() {
		return "Buy";
	}

	public String getSellVerb() {
		return "Sell";
	}

	public boolean isFreeTransfer() {
		return false;
	}

	public boolean isEnabled(CoreUIAPI ui) {
		return ui.getTradeMode() == CoreUITradeMode.OPEN || isBlackMarket();
		//return true;
	}
	public OnClickAction getOnClickAction(CoreUIAPI ui) {
		return OnClickAction.OPEN_SUBMARKET;
	}
	public String getDialogText(CoreUIAPI ui) {
		return null;
	}
	public Highlights getDialogTextHighlights(CoreUIAPI ui) {
		return null;
	}
	public DialogOption [] getDialogOptions(CoreUIAPI ui) {
		return null;
	}
	public String getTooltipAppendix(CoreUIAPI ui) {
		return null;
	}
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		return null;
	}
	

	public void reportPlayerMarketTransaction(PlayerMarketTransaction transaction) {
		if (!isParticipatesInEconomy()) return;
		
		for (CargoStackAPI stack : transaction.getSold().getStacksCopy()) {
			if (stack.isCommodityStack()) {
				float qty = stack.getSize();
				CommodityOnMarketAPI com = market.getCommodityData(stack.getCommodityId());
				com.addToStockpile(qty);
			}
		}
		for (CargoStackAPI stack : transaction.getBought().getStacksCopy()) {
			if (stack.isCommodityStack()) {
				float qty = stack.getSize();
				CommodityOnMarketAPI com = market.getCommodityData(stack.getCommodityId());
				com.removeFromStockpile(qty);
				//System.out.println("New average: " + com.getAverageStockpile());
			}
		}
	}

	public boolean isBlackMarket() {
		//return false;
		return market.getFaction().isHostileTo(submarket.getFaction());
	}

	public boolean isOpenMarket() {
		return false;
	}
	
	public boolean isParticipatesInEconomy() {
		return true;
	}
	

	public boolean isIllegalOnSubmarket(String commodityId, TransferAction action) {
		if (market.hasCondition(Conditions.FREE_PORT)) return false;
		//return market.isIllegal(commodityId); 
		return submarket.getFaction().isIllegal(commodityId); 
	}

	public boolean isIllegalOnSubmarket(CargoStackAPI stack, TransferAction action) {
		if (!stack.isCommodityStack()) return false;
		return isIllegalOnSubmarket((String) stack.getData(), action);
	}
	
	public String getIllegalTransferText(CargoStackAPI stack, TransferAction action) {
		return "Illegal to trade on the " + submarket.getNameOneLine().toLowerCase() + " here";
	}
	
	public boolean isIllegalOnSubmarket(FleetMemberAPI member, TransferAction action) {
		return false;
	}
	
	public String getIllegalTransferText(FleetMemberAPI member, TransferAction action) {
		//return "Illegal to trade on the " + submarket.getNameOneLine().toLowerCase() + " here";
		if (action == TransferAction.PLAYER_BUY) {
			return "Illegal to buy"; // this shouldn't happen
		} else {
			return "Illegal to sell";
		}
	}
	
	
	protected void addWeaponsBasedOnMarketSize(int baseCount, int baseWings, int maxTier, WeightedRandomPicker<FactionAPI> factionPicker) {
		int marketSize = market.getSize();
		addWeaponsForRolePicks(baseCount, baseWings, maxTier, factionPicker,
				ShipRoles.COMBAT_SMALL, 
				ShipRoles.COMBAT_SMALL, 
				ShipRoles.COMBAT_SMALL, 
				ShipRoles.ESCORT_SMALL, 
				ShipRoles.FREIGHTER_SMALL, 
				ShipRoles.COMBAT_FREIGHTER_SMALL, 
				ShipRoles.CARRIER_SMALL);
		if (marketSize >= 2) {
			addWeaponsForRolePicks(baseCount, baseWings, maxTier, factionPicker,
					ShipRoles.COMBAT_MEDIUM,
					ShipRoles.COMBAT_MEDIUM, 
					ShipRoles.ESCORT_MEDIUM, 
					ShipRoles.ESCORT_MEDIUM, 
					ShipRoles.FREIGHTER_MEDIUM, 
					ShipRoles.COMBAT_FREIGHTER_MEDIUM, 
					ShipRoles.CARRIER_MEDIUM);
		}
		if (marketSize >= 4) {
			addWeaponsForRolePicks(baseCount, baseWings, maxTier, factionPicker,
					ShipRoles.CARRIER_LARGE,
					ShipRoles.FREIGHTER_LARGE,
					ShipRoles.COMBAT_FREIGHTER_LARGE,
					ShipRoles.COMBAT_LARGE,
					ShipRoles.COMBAT_LARGE);
		}
		if (marketSize >= 5) {
			addWeaponsForRolePicks(baseCount, baseWings, maxTier, factionPicker,
					ShipRoles.CARRIER_LARGE,
					ShipRoles.COMBAT_CAPITAL,
					ShipRoles.COMBAT_CAPITAL);
		}

	}
	
	protected void addWeaponsForRolePicks(int maxCount, int maxWings, int maxTier, WeightedRandomPicker<FactionAPI> factionPicker, String ... roles) {
		WeightedRandomPicker<String> picker = new WeightedRandomPicker<String>();
		if (!Global.getSettings().isDevMode() || true) {
			picker.setRandom(itemGenRandom);
		}
		
		for (String role : roles) {
			List<WeaponSpecAPI> weapons = getWeaponsOnRolePick(role, factionPicker);
			for (WeaponSpecAPI spec : weapons) {
				if (spec.getTier() > maxTier) continue;
				picker.add(spec.getWeaponId(), spec.getSize().ordinal() + 1);
			}
		}
		
		CargoAPI cargo = getCargo();
		int picks = 0;
		while (!picker.isEmpty() && picks < maxCount) {
			String id = picker.pick();
			cargo.addWeapons(id, 2);
			picker.remove(id);
			picks++;
		}
		
		picker.clear();
		for (String role : roles) {
			List<FighterWingSpecAPI> wings = getWingsOnRolePick(role, factionPicker);
			for (FighterWingSpecAPI spec : wings) {
				if (spec.getTier() > maxTier) continue;
				picker.add(spec.getId(), 1);
			}
		}
		
		picks = 0;
		while (!picker.isEmpty() && picks < maxWings) {
			String id = picker.pick();
			cargo.addItems(CargoItemType.FIGHTER_CHIP, id, 1);
			picker.remove(id);
			picks++;
		}
	}
	
	protected List<WeaponSpecAPI> getWeaponsOnRolePick(String role, WeightedRandomPicker<FactionAPI> factionPicker) {
		float qf = market.getShipQualityFactor();
		
		List<WeaponSpecAPI> result = new ArrayList<WeaponSpecAPI>();
		
		FactionAPI faction = submarket.getFaction();
		if (factionPicker != null && !factionPicker.isEmpty()) {
			faction = factionPicker.pick();
		}
		List<ShipRolePick> picks = faction.pickShip(role, qf, itemGenRandom);
		for (ShipRolePick pick : picks) {
			FleetMemberType type = FleetMemberType.SHIP;
			if (pick.isFighterWing()) {
				//type = FleetMemberType.FIGHTER_WING;
				continue;
			}
			
			FleetMemberAPI member = Global.getFactory().createFleetMember(type, pick.variantId);
			
			for (String slotId : member.getVariant().getFittedWeaponSlots()) {
				WeaponSlotAPI slot = member.getVariant().getSlot(slotId);
				if (slot.isBuiltIn() || slot.isSystemSlot() || slot.isDecorative()) continue;
				WeaponSpecAPI spec = member.getVariant().getWeaponSpec(slotId);
				if (spec.getAIHints().contains(AIHints.SYSTEM)) continue;
				result.add(spec);
			}
		}
		return result;
	}
	
	protected List<FighterWingSpecAPI> getWingsOnRolePick(String role, WeightedRandomPicker<FactionAPI> factionPicker) {
		float qf = market.getShipQualityFactor();
		
		List<FighterWingSpecAPI> result = new ArrayList<FighterWingSpecAPI>();
		
		FactionAPI faction = submarket.getFaction();
		if (factionPicker != null && !factionPicker.isEmpty()) {
			faction = factionPicker.pick();
		}
		List<ShipRolePick> picks = faction.pickShip(role, qf, itemGenRandom);
		for (ShipRolePick pick : picks) {
			FleetMemberType type = FleetMemberType.SHIP;
			if (pick.isFighterWing()) continue;
			
			FleetMemberAPI member = Global.getFactory().createFleetMember(type, pick.variantId);
			for (String wingId : member.getVariant().getNonBuiltInWings()) {
				FighterWingSpecAPI spec = Global.getSettings().getFighterWingSpec(wingId);
				if (spec.getTags().contains(Tags.WING_NO_SELL)) continue;
				result.add(spec);
			}
		}
		return result;
	}
	
	protected void addRandomWeapons(int max, int maxTier) {
		CargoAPI cargo = getCargo();
		List<String> weaponIds = Global.getSector().getAllWeaponIds();
		
		WeightedRandomPicker<String> picker = new WeightedRandomPicker<String>();
		if (!Global.getSettings().isDevMode() || true) {
			picker.setRandom(itemGenRandom);
		}
		
		//float qualityFactor = market.getShipQualityFactor(); 
		for (String id : weaponIds) {
			WeaponSpecAPI spec = Global.getSettings().getWeaponSpec(id);
			if (spec.getTier() <= maxTier) {
				//float weaponQuality = 0.33f * (float) spec.getTier();
				//float qualityDiff = Math.abs(qualityFactor - weaponQuality);
				float qualityDiff = 0f;
				float f = Math.max(0, 1f - qualityDiff);
				float weight = Math.max(0.05f, f * f);
				if (id.equals("phasebeam")) {
					weight *= 50f;
				}
				if (id.equals("lightag")) {
					weight *= 150f;
				}
				if (id.equals("irpulse")) {
					weight *= 100f;
				}
				if (id.equals("sabotpod")) {
					weight *= 15f;
				}
				if (id.equals("autopulse")) {
					weight *= 5f;
				}
				if (id.equals("taclaser")) {
					weight *= 75f;
				}
				if (id.equals("heavymg")) {
					weight *= 50f;
				}
				if (id.equals("heavymortar")) {
					weight *= 75f;
				}
				if (id.equals("chaingun")) {
					weight *= 40f;
				}
				if (id.equals("lightneedler")) {
					weight *= 3f;
				}
				if (id.equals("railgun")) {
					weight *= 7f;
				}
				if (id.equals("hammerrack")) {
					weight *= 15f;
				}
				
				weight *= spec.getRarity();
				
				picker.add(spec.getWeaponId(), weight);
			}
		}
		if (!picker.isEmpty()) {
			for (int i = 0; i < max; i++) {
				String weaponId = picker.pick();
				int quantity = 2;
				cargo.addWeapons(weaponId, quantity);
			}	
		}
		
//		for (int i = 0; i < max; i++) {
//			String weaponId = (String) weaponIds.get((int) (weaponIds.size() * Math.random()));
//			//int quantity = (int)(Math.random() * 4f + 2f);
//			int quantity = 1;
//			cargo.addWeapons(weaponId, quantity);
//		}
	}
	
	protected void addRandomWings(int max, int maxTier) {
		CargoAPI cargo = getCargo();
		List<String> wingIds = Global.getSector().getAllFighterWingIds();
		
		WeightedRandomPicker<String> picker = new WeightedRandomPicker<String>();
		if (!Global.getSettings().isDevMode() || true) {
			picker.setRandom(itemGenRandom);
		}
		
		//float qualityFactor = market.getShipQualityFactor(); 
		for (String id : wingIds) {
			FighterWingSpecAPI spec = Global.getSettings().getFighterWingSpec(id);
			if (spec.getTags().contains(Tags.WING_NO_SELL)) continue;
			if (spec.getTier() <= maxTier) {
				float weight = 1f;
				if (id.equals("xyphos_wing")) {
					weight *= 2f;
				}
				if (id.equals("thunder_wing")) {
					weight *= 2f;
				}
				weight *= spec.getRarity();
				
				picker.add(spec.getId(), weight);
			}
		}
		if (!picker.isEmpty()) {
			for (int i = 0; i < max; i++) {
				String wingId = picker.pick();
				int quantity = 1;
				cargo.addItems(CargoItemType.FIGHTER_CHIP, wingId, quantity);
			}	
		}
	}
	
	protected void pruneWeapons(float mult) {
		CargoAPI cargo = getCargo();
		for (CargoStackAPI stack : cargo.getStacksCopy()) {
			if (stack.isWeaponStack() || stack.isFighterWingStack()) {
				float qty = stack.getSize();
				if (qty <= 1) {
					if (itemGenRandom.nextFloat() > mult) {
						cargo.removeItems(stack.getType(), stack.getData(), 1);
					}
				} else {
					cargo.removeItems(stack.getType(), stack.getData(), Math.round(qty * (1f - mult)));
				}
			}
		}
	}
	
	protected void addShip(String variantOrWingId) {
		FleetMemberAPI member = null;
		if (variantOrWingId.endsWith("_wing")) {
			member = Global.getFactory().createFleetMember(FleetMemberType.FIGHTER_WING, variantOrWingId);
		} else {
			member = Global.getFactory().createFleetMember(FleetMemberType.SHIP, variantOrWingId);
		}
		member.getRepairTracker().setMothballed(true);
		getCargo().getMothballedShips().addFleetMember(member);
	}
	

	protected void addShipsForRoles(int maxTotal, WeightedRandomPicker<String> rolePicker, WeightedRandomPicker<FactionAPI> factionPicker) {
		if (rolePicker.isEmpty()) return;
		
		//if (!Global.getSettings().isDevMode()) {
			rolePicker.setRandom(itemGenRandom);
			if (factionPicker != null) {
				factionPicker.setRandom(itemGenRandom);
			}
		//}
		
		int curr = getCargo().getMothballedShips().getMembersListCopy().size();
		int toAdd = maxTotal - curr;
		int added = 0;
		while (added < toAdd) {
			String role = rolePicker.pick();
			addShipsForRole(role, rolePicker.getRandom(), factionPicker);
			added++;
		}
	}
	
	protected void addShipsForRole(String role, Random random, WeightedRandomPicker<FactionAPI> factionPicker) {
		float qf = market.getShipQualityFactor();
		FactionAPI faction = submarket.getFaction();
		if (factionPicker != null && !factionPicker.isEmpty()) {
			faction = factionPicker.pick();
		}
		List<ShipRolePick> picks = null;
		if (role.equals(ShipRoles.MARKET_RANDOM_SHIPS)) {
			WeightedRandomPicker<ShipRolePick> picker = new WeightedRandomPicker<ShipRolePick>();
			addVariantIfExists(picker, "doom_Strike", 1f);
			addVariantIfExists(picker, "harbinger_Strike", 3f);
			addVariantIfExists(picker, "shade_Assault", 6f);
			addVariantIfExists(picker, "afflictor_Strike", 6f);
			picks = new ArrayList<ShipRolePick>();
			ShipRolePick pick = picker.pick();
			if (pick == null) return;
			picks.add(pick);
		} else {
			picks = faction.pickShip(role, qf, random);
		}
		
		if (picks == null) return;
		
		
		for (ShipRolePick pick : picks) {
			FleetMemberType type = FleetMemberType.SHIP;
			String variantId = pick.variantId;
			if (pick.isFighterWing()) {
				type = FleetMemberType.FIGHTER_WING;
			} else {
				FleetMemberAPI member = Global.getFactory().createFleetMember(type, pick.variantId);
				variantId = member.getHullId() + "_Hull";
				//variantId = "vigilance_Hull";
			}
			FleetMemberAPI member = Global.getFactory().createFleetMember(type, variantId);
			member.getRepairTracker().setMothballed(true);
			getCargo().getMothballedShips().addFleetMember(member);
		}
	}

	protected void addVariantIfExists(WeightedRandomPicker<ShipRolePick> picker, String variantId, float weight) {
		try {
			Global.getSettings().getVariant(variantId);
			ShipRolePick pick = new ShipRolePick(variantId);
			picker.add(pick, weight);
		} catch (Throwable t) {}
	}
	
	protected void pruneShips(float mult) {
		CargoAPI cargo = getCargo();
		FleetDataAPI data = cargo.getMothballedShips();
		for (FleetMemberAPI member : data.getMembersListCopy()) {
			if (itemGenRandom.nextFloat() > mult) {
				data.removeFleetMember(member);
			}
		}
	}
	
	
	protected void addHullMods(int maxTier, int num) {
		//float p = Global.getSettings().getFloat("sellHullmodProb");
		
		CargoAPI cargo = getCargo();
		for (CargoStackAPI stack : cargo.getStacksCopy()) {
			if (stack.isModSpecStack()) {
				cargo.removeStack(stack);
			}
		}
		
		WeightedRandomPicker<HullModSpecAPI> picker = new WeightedRandomPicker<HullModSpecAPI>(itemGenRandom);
		for (String id : submarket.getFaction().getHullMods()) {
			//if (Global.getSector().getCharacterData().knowsHullMod(id)) continue;
			HullModSpecAPI spec = Global.getSettings().getHullModSpec(id);
			if (spec.isHidden()) continue;
			if (spec.isAlwaysUnlocked()) continue;
			if (spec.getTier() > maxTier) continue;
			picker.add(spec, spec.getRarity());
		}
		
		for (int i = 0; i < num; i++) {
			HullModSpecAPI pick = picker.pickAndRemove();
			if (pick == null) continue;
			
			String id = pick.getId();
			if (cargoAlreadyHasMod(id)) continue;
			cargo.addItems(CargoItemType.MOD_SPEC, id, 1);
		}
		
//		for (String id : submarket.getFaction().getHullMods()) {
//			if (Global.getSector().getCharacterData().knowsHullMod(id)) {
//				removeModFromCargo(id);
//				continue;
//			}
//			if (cargoAlreadyHasMod(id)) continue;
//			
//			HullModSpecAPI spec = Global.getSettings().getHullModSpec(id);
//			if (itemGenRandom.nextFloat() > p * spec.getRarity()) continue;
//
//			if (spec.isHidden()) continue;
//			if (spec.isAlwaysUnlocked()) continue;
//			if (spec.getTier() > maxTier) continue;
//			
//			cargo.addItems(CargoItemType.MOD_SPEC, id, 1);
//		}
	}
	
	protected boolean removeModFromCargo(String id) {
		CargoAPI cargo = getCargo();
		for (CargoStackAPI stack : cargo.getStacksCopy()) {
			if (stack.isModSpecStack() && stack.getData().equals(id)) {
				cargo.removeStack(stack);
			}
		}
		return false;
	}
	protected boolean cargoAlreadyHasMod(String id) {
		CargoAPI cargo = getCargo();
		for (CargoStackAPI stack : cargo.getStacksCopy()) {
			if (stack.isModSpecStack() && stack.getData().equals(id)) return true;
		}
		return false;
	}
	

	public Highlights getIllegalTransferTextHighlights(CargoStackAPI stack, TransferAction action) {
		return null;
	}

	public Highlights getIllegalTransferTextHighlights(FleetMemberAPI member, TransferAction action) {
		return null;
	}

	public float getMinCargoUpdateInterval() {
		return minCargoUpdateInterval;
	}

	public void setMinCargoUpdateInterval(float minCargoUpdateInterval) {
		this.minCargoUpdateInterval = minCargoUpdateInterval;
	}

	public float getSinceLastCargoUpdate() {
		return sinceLastCargoUpdate;
	}

	public void setSinceLastCargoUpdate(float sinceLastCargoUpdate) {
		this.sinceLastCargoUpdate = sinceLastCargoUpdate;
	}
	
}







