package com.fs.starfarer.api.impl.campaign.submarkets;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.impl.campaign.CoreCampaignPluginImpl;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class BlackMarketPlugin extends BaseSubmarketPlugin {
	
	public static Logger log = Global.getLogger(BlackMarketPlugin.class);
	
	public void init(SubmarketAPI submarket) {
		super.init(submarket);
	}


	public void updateCargoPrePlayerInteraction() {
		
		if (!okToUpdateCargo()) return;
		sinceLastCargoUpdate = 0f;
		
		updateEconomicCommoditiesInCargo(true);
		
		float stability = market.getStabilityValue();
		CargoAPI cargo = getCargo();
		
		pruneWeapons(0.5f);
		
		WeightedRandomPicker<FactionAPI> factionPicker = new WeightedRandomPicker<FactionAPI>();
		factionPicker.add(market.getFaction(), 11f - stability);
		factionPicker.add(Global.getSector().getFaction(Factions.INDEPENDENT), 4f);
		factionPicker.add(submarket.getFaction(), 6f);
		addWeaponsBasedOnMarketSize(3, 2, 3, factionPicker);
		
		addRandomWeapons(Math.max(1, market.getSize() - 3), 3);
		addRandomWings(Math.max(1, market.getSize() - 5), 3);
	
		addShips();
		addHullMods(4, 1 + itemGenRandom.nextInt(3));
		cargo.sort();
		
	}
	
	public float getDesiredAboveDemandCommodityQuantity(CommodityOnMarketAPI com) {
		boolean illegal = market.isIllegal(com.getId());
		if (illegal) return com.getAboveDemandStockpile();
		
		float blackMarketLegalFraction = 1f - 0.09f * market.getStabilityValue();
		return com.getAboveDemandStockpile() * blackMarketLegalFraction;
	}
	
	public float getDesiredBelowDemandCommodityQuantity(CommodityOnMarketAPI com) {
		boolean illegal = market.isIllegal(com.getId());
		
		float blackMarketLegalFraction = 0.25f - 0.02f * market.getStabilityValue();
		float blackMarketIllegalFraction = 0.5f - 0.025f * market.getStabilityValue();
		
		float fraction = blackMarketLegalFraction;
		if (illegal) {
			fraction = blackMarketIllegalFraction;
		}
		return Math.max(0, com.getStockpile() - com.getAboveDemandStockpile()) * fraction;
	}
	
	private void addShips() {
		int marketSize = market.getSize();
		
		pruneShips(0.75f);
		
		WeightedRandomPicker<String> rolePicker = new WeightedRandomPicker<String>();
		rolePicker.add(ShipRoles.CIV_RANDOM, 5f);
		rolePicker.add(ShipRoles.FREIGHTER_SMALL, 3f);
		rolePicker.add(ShipRoles.TANKER_SMALL, 3f);
		rolePicker.add(ShipRoles.COMBAT_FREIGHTER_SMALL, 15f);
		rolePicker.add(ShipRoles.COMBAT_SMALL, 15f);
		rolePicker.add(ShipRoles.COMBAT_MEDIUM, 15f);
		rolePicker.add(ShipRoles.LINER_SMALL, 1f);
		rolePicker.add(ShipRoles.MARKET_RANDOM_SHIPS, 1f);
		
		if (marketSize >= 4) {
			rolePicker.add(ShipRoles.FREIGHTER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.TANKER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_MEDIUM, 10f);
			rolePicker.add(ShipRoles.COMBAT_MEDIUM, 10f);
			rolePicker.add(ShipRoles.CARRIER_SMALL, 10f);
			rolePicker.add(ShipRoles.LINER_MEDIUM, 1f);
		}
		
		if (marketSize >= 6) {
			rolePicker.add(ShipRoles.FREIGHTER_LARGE, 3f);
			rolePicker.add(ShipRoles.TANKER_LARGE, 3f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_LARGE, 5f);
			rolePicker.add(ShipRoles.COMBAT_LARGE, 5f);
			rolePicker.add(ShipRoles.CARRIER_MEDIUM, 10f);
			rolePicker.add(ShipRoles.LINER_LARGE, 1f);
		}
		
		WeightedRandomPicker<FactionAPI> factionPicker = new WeightedRandomPicker<FactionAPI>();
		float stability = market.getStabilityValue();
		factionPicker.add(market.getFaction(), 11f - stability);
		factionPicker.add(Global.getSector().getFaction(Factions.INDEPENDENT), 4f);
		factionPicker.add(submarket.getFaction(), 6f);
		
		addShipsForRoles(marketSize * 2, rolePicker, factionPicker);
	}
	

	@Override
	public boolean isIllegalOnSubmarket(CargoStackAPI stack, TransferAction action) {
		return false;
	}


	@Override
	public boolean isIllegalOnSubmarket(String commodityId, TransferAction action) {
		return false;
	}
	
	public float getTariff() {
		return 0f;
	}


	@Override
	public boolean isBlackMarket() {
		return true;
	}
	
	
	public String getTooltipAppendix(CoreUIAPI ui) {
		if (isEnabled(ui)) {
//			CampaignEventManagerAPI manager = Global.getSector().getEventManager();
//			EventProbabilityAPI ep = manager.getProbability(Events.INVESTIGATION_SMUGGLING, market);
//			float p = ep.getProbability();
			
			float p = CoreCampaignPluginImpl.computeSmugglingSuspicionLevel(market);
			if (p < 0.05f) return "Suspicion level: none";
			
			if (p < 0.1f) {
				return "Suspicion level: minimal";
			}
			if (p < 0.2f) {
				return "Suspicion level: medium";
			}
			if (p < 0.3f) {
				return "Suspicion level: high";
			}
			if (p < 0.5f) {
				return "Suspicion level: very high";
			}
			return "Suspicion level: extreme";
		}

		return null;
	}
	
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		String appendix = getTooltipAppendix(ui);
		if (appendix == null) return null;
		
		Highlights h = new Highlights();
		h.setText(appendix);
		h.setColors(Misc.getNegativeHighlightColor());
		return h;
	}
}



