package com.fs.starfarer.api.impl.campaign.submarkets;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CargoStackAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.RepLevel;
import com.fs.starfarer.api.campaign.CampaignUIAPI.CoreUITradeMode;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.loading.FighterWingSpecAPI;
import com.fs.starfarer.api.loading.HullModSpecAPI;
import com.fs.starfarer.api.loading.WeaponSpecAPI;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class MilitarySubmarketPlugin extends BaseSubmarketPlugin {
	
	public static Logger log = Global.getLogger(MilitarySubmarketPlugin.class);
	
	public void init(SubmarketAPI submarket) {
		super.init(submarket);
	}

	public void updateCargoPrePlayerInteraction() {
		
		
		if (!okToUpdateCargo()) return;
		sinceLastCargoUpdate = 0f;
		
		updateEconomicCommoditiesInCargo(false);
		
		CargoAPI cargo = getCargo();
		
		pruneWeapons(0.5f);
		addWeaponsBasedOnMarketSize(10, 3, 3, null);
		addRandomWeapons(Math.max(1, market.getSize() - 3), 3);
		addRandomWings(Math.max(1, market.getSize() - 4), 3);
		
		addShips();	
		addHullMods(4, 2 + itemGenRandom.nextInt(4));
		cargo.sort();
	}
	
	public float getDesiredAboveDemandCommodityQuantity(CommodityOnMarketAPI com) {
		if (com.getCommodity().hasTag(Commodities.TAG_MILITARY)) {
			return com.getAboveDemandStockpile(); 
		}
		return 0f;
	}
	
	public float getDesiredBelowDemandCommodityQuantity(CommodityOnMarketAPI com) {
		boolean fuel = Commodities.FUEL.equals(com.getId());
		boolean supplies = Commodities.SUPPLIES.equals(com.getId());
		boolean crew = Commodities.CREW.equals(com.getId());
		if (!fuel && !supplies && !crew) return 0f;
		
		float fraction = 0.2f;
		return Math.max(0, com.getStockpile() - com.getAboveDemandStockpile()) * fraction;
	}
	
	
	private void addShips() {
		int marketSize = market.getSize();
		
		pruneShips(0.75f);
		
		WeightedRandomPicker<String> rolePicker = new WeightedRandomPicker<String>();
		rolePicker.add(ShipRoles.FREIGHTER_SMALL, 3f);
		rolePicker.add(ShipRoles.TANKER_SMALL, 3f);
		rolePicker.add(ShipRoles.COMBAT_SMALL, 15f);
		rolePicker.add(ShipRoles.ESCORT_SMALL, 10f);
		rolePicker.add(ShipRoles.COMBAT_MEDIUM, 15f);
		rolePicker.add(ShipRoles.CARRIER_SMALL, 3f);
		rolePicker.add(ShipRoles.MARKET_RANDOM_SHIPS, 1f);
		
		if (marketSize >= 4) {
			rolePicker.add(ShipRoles.FREIGHTER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.TANKER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.COMBAT_MEDIUM, 10f);
			rolePicker.add(ShipRoles.ESCORT_MEDIUM, 10f);
			rolePicker.add(ShipRoles.CARRIER_SMALL, 10f);
			rolePicker.add(ShipRoles.COMBAT_LARGE, 10f);
			rolePicker.add(ShipRoles.COMBAT_CAPITAL, 3f);
		}
		
		if (marketSize >= 5) {
			rolePicker.add(ShipRoles.FREIGHTER_LARGE, 3f);
			rolePicker.add(ShipRoles.TANKER_LARGE, 3f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_LARGE, 5f);
			rolePicker.add(ShipRoles.COMBAT_LARGE, 10f);
			rolePicker.add(ShipRoles.COMBAT_CAPITAL, 5f);
			rolePicker.add(ShipRoles.CARRIER_MEDIUM, 10f);
			rolePicker.add(ShipRoles.CARRIER_LARGE, 3f);
		}
		
		addShipsForRoles(2 + marketSize * 2, rolePicker, null);
		
		if (marketSize >= 4) {
			boolean hasCapital = false;
			for (FleetMemberAPI member : getCargo().getMothballedShips().getMembersListCopy()) {
				if (member.isCapital()) {
					hasCapital = true;
					break;
				}
			}
			
			if (!hasCapital) {
				rolePicker.clear();
				rolePicker.add(ShipRoles.COMBAT_CAPITAL, 1f);
				addShipsForRoles(getCargo().getMothballedShips().getMembersListCopy().size() + 1, rolePicker, null);
			}
		}
	}

	
	@Override
	public String getName() {
		if (submarket.getFaction().getId().equals(Factions.LUDDIC_CHURCH)) {
			return "Knights of Ludd";
		}
		return Misc.ucFirst(submarket.getFaction().getDisplayName()) + "\n" + "Military";
	}

	protected boolean requiresCommission(RepLevel req) {
		if (!submarket.getFaction().getCustomBoolean(Factions.CUSTOM_OFFERS_COMMISSIONS)) return false;
		
		if (req.isAtWorst(RepLevel.WELCOMING)) return true;
		return false;
	}
	
	protected boolean hasCommission() {
		return submarket.getFaction().getId().equals(Misc.getCommissionFactionId());
	}
	
	public boolean isIllegalOnSubmarket(String commodityId, TransferAction action) {
		boolean illegal = submarket.getFaction().isIllegal(commodityId);
		RepLevel req = getRequiredLevelAssumingLegal(commodityId, action);
		
		if (req == null) return illegal;
		
		RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
		boolean legal = level.isAtWorst(req);
		if (requiresCommission(req)) {
			legal &= hasCommission();
		}
		return !legal;
	}

	public boolean isIllegalOnSubmarket(CargoStackAPI stack, TransferAction action) {
		if (stack.isCommodityStack()) {
			return isIllegalOnSubmarket((String) stack.getData(), action);
		}
		
		RepLevel req = getRequiredLevelAssumingLegal(stack, action);
		if (req == null) return false;
		
		RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
		
		boolean legal = level.isAtWorst(req);
		if (requiresCommission(req)) {
			legal &= hasCommission();
		}
		
		return !legal;
	}
	
	public String getIllegalTransferText(CargoStackAPI stack, TransferAction action) {
		RepLevel req = getRequiredLevelAssumingLegal(stack, action);

		if (req != null) {
			if (requiresCommission(req)) {
				return "Req: " +
						submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase() + ", " +
						" commission";
			}
			return "Req: " + 
					submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase();
		}
		
		return "Illegal to trade in " + stack.getDisplayName() + " here";
	}
	

	public Highlights getIllegalTransferTextHighlights(CargoStackAPI stack, TransferAction action) {
		RepLevel req = getRequiredLevelAssumingLegal(stack, action);
		if (req != null) {
			Color c = Misc.getNegativeHighlightColor();
			Highlights h = new Highlights();
			RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
			if (!level.isAtWorst(req)) {
				h.append(submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase(), c);
			}
			if (requiresCommission(req) && !hasCommission()) {
				h.append("commission", c);
			}
			return h;
		}
		return null;
	}
	
	private RepLevel getRequiredLevelAssumingLegal(CargoStackAPI stack, TransferAction action) {
		int tier = -1;
		if (stack.isWeaponStack()) {
			WeaponSpecAPI spec = stack.getWeaponSpecIfWeapon();
			tier = spec.getTier();
		} else if (stack.isModSpecStack()) {
			HullModSpecAPI spec = stack.getHullModSpecIfHullMod();
			tier = spec.getTier();
		} else if (stack.isFighterWingStack()) {
			FighterWingSpecAPI spec = stack.getFighterWingSpecIfWing();
			tier = spec.getTier();
		}
		
		if (tier >= 0) {
			if (action == TransferAction.PLAYER_BUY) {
				switch (tier) {
				case 0: return RepLevel.FAVORABLE;
				case 1: return RepLevel.WELCOMING;
				case 2: return RepLevel.FRIENDLY;
				case 3: return RepLevel.COOPERATIVE;
				}
			}
			return RepLevel.VENGEFUL;
		}
		
		if (!stack.isCommodityStack()) return null;
		return getRequiredLevelAssumingLegal((String) stack.getData(), action);
	}
	
	private RepLevel getRequiredLevelAssumingLegal(String commodityId, TransferAction action) {
		if (action == TransferAction.PLAYER_SELL) {
			//return null;
			return RepLevel.VENGEFUL;
		}
		
		CommodityOnMarketAPI com = market.getCommodityData(commodityId);
		boolean isMilitary = com.getCommodity().getTags().contains(Commodities.TAG_MILITARY);
		if (isMilitary) {
			if (com.isPersonnel()) {
				return RepLevel.COOPERATIVE;
			}
			return RepLevel.FAVORABLE;
		}
		return null;
	}
	
	public boolean isIllegalOnSubmarket(FleetMemberAPI member, TransferAction action) {
		RepLevel req = getRequiredLevelAssumingLegal(member, action);
		if (req == null) return false;
		
		RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
		
		boolean legal = level.isAtWorst(req);
		if (requiresCommission(req)) {
			legal &= hasCommission();
		}
		
		return !legal;
	}
	
	public String getIllegalTransferText(FleetMemberAPI member, TransferAction action) {
		RepLevel req = getRequiredLevelAssumingLegal(member, action);
		if (req != null) {
			String str = "";
			RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
			if (!level.isAtWorst(req)) {
				str += "Req: " + submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase();				
			}
			if (requiresCommission(req) && !hasCommission()) {
				if (!str.isEmpty()) str += "\n";
				str += "Req: " + submarket.getFaction().getDisplayName() + " - " + "commission";
			}
			return str;
//			if (requiresCommission(req)) {
//				return //"Requires:\n" +
//						"Req: " + submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase() + "\n" +
//						"Req: " + submarket.getFaction().getDisplayName() + " - " + "commission";
//			}
//			return "Requires: " + 
//					submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase();
		}
		
		if (action == TransferAction.PLAYER_BUY) {
			return "Illegal to buy"; // this shouldn't happen
		} else {
			return "Illegal to sell";
		}
	}

	public Highlights getIllegalTransferTextHighlights(FleetMemberAPI member, TransferAction action) {
		RepLevel req = getRequiredLevelAssumingLegal(member, action);
		if (req != null) {
			Color c = Misc.getNegativeHighlightColor();
			Highlights h = new Highlights();
			RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
			if (!level.isAtWorst(req)) {
				h.append("Req: " + submarket.getFaction().getDisplayName() + " - " + req.getDisplayName().toLowerCase(), c);
			}
			if (requiresCommission(req) && !hasCommission()) {
				h.append("Req: " + submarket.getFaction().getDisplayName() + " - commission", c);
			}
			return h;
		}
		return null;
	}
	
	private RepLevel getRequiredLevelAssumingLegal(FleetMemberAPI member, TransferAction action) {
		if (action == TransferAction.PLAYER_BUY) {
			int fp = member.getFleetPointCost();
			HullSize size = member.getHullSpec().getHullSize();
			
			if (size == HullSize.CAPITAL_SHIP || fp > 15) return RepLevel.COOPERATIVE;
			if (size == HullSize.CRUISER || fp > 10) return RepLevel.FRIENDLY;
			if (size == HullSize.DESTROYER || fp > 5) return RepLevel.WELCOMING;
			return RepLevel.FAVORABLE;
		}
		return null;
	}
	
	
	
	private RepLevel minStanding = RepLevel.FAVORABLE;
	public boolean isEnabled(CoreUIAPI ui) {
		//if (mode == CoreUITradeMode.OPEN) return false;
		if (ui.getTradeMode() == CoreUITradeMode.SNEAK) return false;
		
		RepLevel level = submarket.getFaction().getRelationshipLevel(Global.getSector().getFaction(Factions.PLAYER));
		return level.isAtWorst(minStanding);
	}
	
	public OnClickAction getOnClickAction(CoreUIAPI ui) {
		return OnClickAction.OPEN_SUBMARKET;
	}
	
	public String getTooltipAppendix(CoreUIAPI ui) {
		if (!isEnabled(ui)) {
			return "Requires: " + submarket.getFaction().getDisplayName() + " - " + minStanding.getDisplayName().toLowerCase();
		}
		if (ui.getTradeMode() == CoreUITradeMode.SNEAK) {
			return "Requires: proper docking authorization";
		}
		return null;
	}
	
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		String appendix = getTooltipAppendix(ui);
		if (appendix == null) return null;
		
		Highlights h = new Highlights();
		h.setText(appendix);
		h.setColors(Misc.getNegativeHighlightColor());
		return h;
	}
	
	
}



