package com.fs.starfarer.api.impl.campaign.submarkets;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.CoreUIAPI;
import com.fs.starfarer.api.campaign.CampaignUIAPI.CoreUITradeMode;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.SubmarketAPI;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.ShipRoles;
import com.fs.starfarer.api.util.Highlights;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class OpenMarketPlugin extends BaseSubmarketPlugin {
	
	public void init(SubmarketAPI submarket) {
		super.init(submarket);
	}


	public void updateCargoPrePlayerInteraction() {
		
		if (!okToUpdateCargo()) return;
		sinceLastCargoUpdate = 0f;
		
		updateEconomicCommoditiesInCargo(false);
		
		CargoAPI cargo = getCargo();

		pruneWeapons(0.5f);
		addWeaponsBasedOnMarketSize(5, 1, 0, null);
		addRandomWeapons(Math.max(1, market.getSize() - 3), 0);
		addRandomWings(Math.max(1, market.getSize() - 5), 0);
		
		addShips();
		addHullMods(1, 1 + itemGenRandom.nextInt(3));
		cargo.sort();
	}
	
	@Override
	public float getDesiredAboveDemandCommodityQuantity(CommodityOnMarketAPI com) {
		boolean illegal = market.isIllegal(com.getId());
		if (illegal) return 0f;
		
		float openMarketLegalFraction = 0.09f * market.getStabilityValue();
		return com.getAboveDemandStockpile() * openMarketLegalFraction; 
	}
	
	@Override
	public float getDesiredBelowDemandCommodityQuantity(CommodityOnMarketAPI com) {
		boolean fuel = Commodities.FUEL.equals(com.getId());
		boolean supplies = Commodities.SUPPLIES.equals(com.getId());
		boolean crew = Commodities.CREW.equals(com.getId());
		if (!fuel && !supplies && !crew) return 0f;
		
		float fraction = 0.1f;
		return Math.max(0, com.getStockpile() - com.getAboveDemandStockpile()) * fraction;
	}
	
	
	private void addShips() {
		int marketSize = market.getSize();
		
		pruneShips(0.75f);
		
		WeightedRandomPicker<String> rolePicker = new WeightedRandomPicker<String>();
		rolePicker.add(ShipRoles.CRIG, 10f);
		rolePicker.add(ShipRoles.TUG, 10f);
		rolePicker.add(ShipRoles.CIV_RANDOM, 10f);
		rolePicker.add(ShipRoles.FREIGHTER_SMALL, 10f);
		rolePicker.add(ShipRoles.TANKER_SMALL, 20f);
		rolePicker.add(ShipRoles.COMBAT_FREIGHTER_SMALL, 5f);
		rolePicker.add(ShipRoles.LINER_SMALL, 1f);
		
		if (marketSize >= 4) {
			rolePicker.add(ShipRoles.FREIGHTER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.TANKER_MEDIUM, 5f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_MEDIUM, 3f);
			rolePicker.add(ShipRoles.LINER_MEDIUM, 0.5f);
		}
		
		if (marketSize >= 6) {
			rolePicker.add(ShipRoles.FREIGHTER_LARGE, 3f);
			rolePicker.add(ShipRoles.TANKER_LARGE, 3f);
			rolePicker.add(ShipRoles.COMBAT_FREIGHTER_LARGE, 2f);
			rolePicker.add(ShipRoles.LINER_LARGE, 1f);
		}
		
		addShipsForRoles(2 + marketSize, rolePicker, null);

		
		if (Global.getSettings().isDevMode() && false) {
			addShip("talon_wing");
			addShip("crig_Standard");
			addShip("crig_Standard");
		}
		
	}


	@Override
	public boolean isOpenMarket() {
		return true;
	}


	@Override
	public String getTooltipAppendix(CoreUIAPI ui) {
		if (ui.getTradeMode() == CoreUITradeMode.SNEAK) {
			return "Requires: proper docking authorization";
		}
		return super.getTooltipAppendix(ui);
	}


	@Override
	public Highlights getTooltipAppendixHighlights(CoreUIAPI ui) {
		if (ui.getTradeMode() == CoreUITradeMode.SNEAK) {
			String appendix = getTooltipAppendix(ui);
			if (appendix == null) return null;
			
			Highlights h = new Highlights();
			h.setText(appendix);
			h.setColors(Misc.getNegativeHighlightColor());
			return h;
		}
		return super.getTooltipAppendixHighlights(ui);
	}
	
	
	
}
